#!/usr/bin/perl

package Benchmark::Params;

use 5.00503;
use strict;

=head1 NAME

Benchmark::Params - Time the execution of a subroutine

=head1 SYNOPSIS

  use Benchmark::Params qw/benchmark benchmark_random/;
  use Benchmark::Params qw/:all/;

  sub example($ $) { .. }

  benchmark(10, \&example, [1, 2], [1, 3], [2, 5], [0, -1]);

  benchmark_random(10, \&example, [1, 2], [1, 3], [2, 5]);

  benchmark_simple(10, \&example, 2, 5);

  sub noargs() { .. }

  benchmark(20, \&noargs);

=head1 DESCRIPTION

The B<Benchmark::Params> module provides three functions,
C<benchmark()>, C<benchmark_random()>, and C<benchmark_simple()>, which time
the execution of a subroutine.  In each case, you provide zero or more lists
of parameters to be passed to the subroutine.

=head1 EXPORT

None by default.

The C<all> tag encompasses all of C<benchmark()>, C<benchmark_random()>,
and C<benchmark_simple()>.

=cut

require Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Exporter);

%EXPORT_TAGS = ( 'all' => [qw(benchmark benchmark_random benchmark_simple)] );

@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

@EXPORT = qw(
	
);

$VERSION = '0.02';

=head1 FUNCTIONS

=over 4

=item benchmark COUNT, SUB, PARAMREF...

Time C<COUNT> executions of C<SUB> and return an array of the times (in
seconds) for each execution.  If any array references are specified, they
are interpreted as lists of arguments to be passed to C<SUB> in the same
order, looping back to the first one if necessary.

=cut

sub benchmark($ $ @)
{
	my ($count, $sub, @params) = @_;

	return benchmark_real($count, $sub, 0, @params);
}

=item benchmark_random COUNT, SUB, PARAMREF...

Time C<COUNT> executions of C<SUB> and return an array of the times (in
seconds) for each execution.  If any array references are specified, they
are interpreted as lists of arguments to be passed to C<SUB> in random
order.

=cut

sub benchmark_random($ $ @)
{
	my ($count, $sub, @params) = @_;

	return benchmark_real($count, $sub, 1, @params);
}

=item benchmark_simple COUNT, SUB, PARAMS

Time C<COUNT> executions of C<SUB> and return an array of the times (in
seconds) for each execution.  This function only passes one set of arguments
to the subroutine - the C<PARAMS> list.

=cut

sub benchmark_simple($ $ @)
{
	my ($count, $sub, @params) = @_;

	return benchmark_real($count, $sub, 0, [ @params ]);
}

=item benchmark_real COUNT, SUB, RANDOM, PARAMREF...

Time C<COUNT> executions of C<SUB> and return an array of the times (in
seconds) for each execution.  If any array references are specified, they
are interpreted as lists of arguments to be passed to C<SUB> either in
random order, if C<RANDOM> has a true value, or in the order supplied.

This routine is not exported; use C<benchmark()>, C<benchmark_random()> or
C<benchmark_simple()> instead.

=cut

sub benchmark_real($ $ $ @)
{
	my ($count, $sub, $random, @params) = @_;
	my ($i, $p, $pcount, $t1, $t2);
	my (@cparams, @res);

	$pcount = @params;
	for ($i = 0; $i < $count; $i++) {
		if ($pcount == 1) {
			@cparams = @{$params[0]};
		} elsif ($pcount) {
			if ($random) {
				$p = $i % $pcount;
			} else {
				$p = int rand $pcount;
			}
			@cparams = @{$params[$p]};
		} else {
			@cparams = ();
		}

		$t1 = times();
		&{$sub}(@cparams);
		$t2 = times();
		push @res, $t2 - $t1;
	}
	return @res;
}

=back

=head1 SEE ALSO

L<Benchmark>, L<times>

=head1 AUTHOR

Peter Pentchev, roam@ringlet.net

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004 by Peter Pentchev

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.5 or,
at your option, any later version of Perl 5 you may have available.

$Ringlet: perl/misc/Benchmark-Params/Params.pm,v 1.7 2004/12/20 08:47:58 roam Exp $

=cut

1;
